import os
import json
import shutil
import random

from termcolor import cprint
import colorama

from pyvoc.check_config import config_dir_path

from pyvoc import pyvoc


import textwrap

colorama.init()

terminal_width = shutil.get_terminal_size().columns


def revise_vocab(group_number):
    print("")
    group_path = os.path.join(config_dir_path(), "group" + str(group_number) + ".json")
    try:
        with open(os.path.join(group_path), "r") as f:
            group = json.load(f)
    except FileNotFoundError:
        pyvoc.stop_loading_animation()
        cprint(
            "group number {} does not exists".format(group_number),
            color="red",
            attrs=["bold"],
        )
        exit()
    words = list(group)
    random.shuffle(words)
    pyvoc.stop_loading_animation()
    print("")
    cprint(" Press <enter> for next. q<enter> to exit ", "yellow", attrs=["bold"])
    print("")
    for i, word in enumerate(words, 1):
        cprint(
            "{}".format(word),
            color="green",
            attrs=["reverse", "bold"],
            end=" " * (15 - len(word)),
        )
        width_left = terminal_width - 24
        sentences = textwrap.wrap(list(group[word].values())[0], width=width_left)
        s_count = 1
        for sentence in sentences:
            if s_count == 1:
                print(sentence)
            else:
                print(" " * (15) + sentence)
            s_count += 1
        print("{}. ".format(i), end="")
        prompt = input("> ")
        if prompt.lower() == "q":
            break
        print(" ")
    print("")
    cprint("END", color="yellow", attrs=["bold", "reverse"])


def validate_group_number(group_number):
    if group_number in [101, 102, 103]:
        path = os.path.join(config_dir_path(), "group101.json")
        if not os.path.isfile(path):
            pyvoc.stop_loading_animation()
            cprint("group{} does not exist".format(101), color="red", attrs=["bold"])
            exit()
        return "custom group"

    if group_number < 1 or group_number > 100:
        pyvoc.stop_loading_animation()
        cprint("Invalid group number. choose from 1-100", color="red", attrs=["bold"])
        exit()
    if group_number < 51:
        path = os.path.join(config_dir_path(), "usergroups.json")
    else:
        path = os.path.join(config_dir_path(), "defaultgroups.json")
    return path


def check_group_path(group_path, group_number):
    if not os.path.isfile(group_path):
        pyvoc.stop_loading_animation()
        cprint(
            "group number {} does not exist".format(group_number),
            color="red",
            attrs=["bold"],
        )
        exit()


def count_words_in_group(path, group_number, no_of_questions):
    with open(path, "r") as f:
        num_of_words_in_group = json.load(f)[str(group_number)]
        if num_of_words_in_group < no_of_questions:
            pyvoc.stop_loading_animation()
            cprint(
                "group number {} does not have enough words".format(group_number),
                color="red",
                attrs=["bold"],
            )
            exit()


def count_words_in_custom_group(group_path, no_of_questions, group_number):
    with open(group_path, "r") as f:
        content = json.load(f)

    no_of_words_in_group = len(list(content.keys()))
    if no_of_questions > no_of_words_in_group:
        pyvoc.stop_loading_animation()
        cprint(
            "group number {} does not have enough words".format(group_number),
            color="red",
            attrs=["bold"],
        )
        exit()


def quiz(group_number, no_of_questions=5):
    print("")
    path = validate_group_number(group_number)
    group_path = os.path.join(config_dir_path(), "group" + str(group_number) + ".json")
    options_path = os.path.join(config_dir_path(), "options.json")
    check_group_path(group_path, group_number)
    if path == "custom group":
        count_words_in_custom_group(group_path, no_of_questions, group_number)
    else:
        count_words_in_group(path, group_number, no_of_questions)
    result = {}
    word_definition = {}
    with open(group_path, "r") as f:
        group_content = json.load(f)
    word_list = random.sample(list(group_content), no_of_questions)
    for word in word_list:
        _ = group_content[word]
        refined_def = _[random.sample(list(_), 1)[0]]
        word_definition[word] = refined_def
    with open(options_path, "r") as f:
        options = json.load(f)
    pyvoc.stop_loading_animation()
    cprint(
        "1 point for every correct answer. q<enter> to exit",
        color="yellow",
        attrs=["bold"],
    )
    print("\n")
    score = 0
    for i in range(no_of_questions):
        cprint(word_list[i], color="white", attrs=["bold", "reverse"])
        correct_option_number = print_options(options, word_definition, word_list[i])
        prompt_input(correct_option_number, word_list[i], score, result, i + 1)
    for word in result:
        if result[word] is True:
            score += 1
    print("")
    cprint("Score: {}/{}".format(score, no_of_questions), color="green", attrs=["bold"])
    if score == no_of_questions:
        cprint("Perfect Score ヽ(´▽`)/", color="yellow", attrs=["bold", "blink"])


def print_options(options, correct_answer, word):
    options_list = []
    for i in random.sample(range(1, 100), 3):
        options_list.append(options[str(i)])
        # print(options[str(i)])
    options_list.append(correct_answer[word])
    random.shuffle(options_list)
    count = 1
    for i, option in enumerate(options_list, 1):
        if option == correct_answer[word]:
            correct_option_number = count
        cprint("[{}]".format(i), color="cyan", end=" ")
        width_left = terminal_width - (3 + len(str(i)))
        sentences = textwrap.wrap(option, width=width_left)
        s_count = 1
        for sentence in sentences:
            if s_count == 1:
                print(sentence)
            else:
                print(" " * (3 + len(str(i))) + sentence)
            s_count += 1

        count += 1
    return correct_option_number


def prompt_input(correct_option_number, word, score, result, question_number):
    while 1:
        prompt = input("{}.> ".format(question_number))
        if prompt.lower() == "q":
            # return
            exit()
        try:
            if not int(prompt) in [1, 2, 3, 4]:
                cprint(
                    "enter a valid integer[1, 2, 3, 4]. q<enter> to exit",
                    color="yellow",
                )
                continue
        except ValueError:
            cprint(
                "enter a valid integer[1, 2, 3, 4]. q<enter> to exit", color="yellow"
            )
            continue
        if int(prompt) == correct_option_number:
            cprint("correct answer", color="green", on_color="on_grey")
            print("")
            result[word] = True
            score += 1
            break
        else:
            cprint("wrong answer", color="red", on_color="on_grey")
            print("")
            result[word] = False
            break



import argparse
import itertools
import threading
import time
import sys


import requests
from termcolor import cprint
import colorama

from pyvoc.check_config import read_config_file, check_config_dir
from pyvoc.dmanager import add_word_to_vocab, list_all_groups
from pyvoc.termoutput import revise_vocab, quiz, terminal_width

import textwrap


colorama.init()
done = False


def parse_dictionary_response(response):
    lexicalEntries = response.json().get("results")[0]["lexicalEntries"]
    parsed_response = dict()
    examples = dict()
    for i in lexicalEntries:
        lexicalCategory = i["lexicalCategory"]
        try:
            definition = i["entries"][0]["senses"][0]["short_definitions"][0]
            try:
                example = i["entries"][0]["senses"][0]["examples"][0]["text"]
            except KeyError:
                example = "None"
        except KeyError:
            try:
                definition = i["entries"][0]["senses"][0]["crossReferenceMarkers"][0]
                try:
                    example = i["entries"][0]["senses"][0]["examples"][0]["text"]
                except KeyError:
                    example = "None"
            except Exception:
                print("No definition found")
        parsed_response[lexicalCategory] = definition
        examples[lexicalCategory] = example
    return parsed_response, examples


def pretty_print_definition(word, parsed_response, examples):
    print("")
    cprint(word + " ", color="cyan", attrs=["bold", "reverse"])
    for key in parsed_response:
        cprint(key + ":", color="green", end=" " * (16 - len(key)))
        width_left = terminal_width - 18
        sentences = textwrap.wrap(parsed_response[key], width=width_left)
        s_count = 1
        for sentence in sentences:
            if s_count == 1:
                print(sentence)
            else:
                print(" " * (17) + sentence)
            s_count += 1
        cprint("example:", color="yellow", end=" " * (16 - len("example")))
        sentences = textwrap.wrap(examples[key], width=width_left)
        s_count = 1
        for sentence in sentences:
            if s_count == 1:
                print(sentence)
            else:
                print(" " * (15) + sentence)
            s_count += 1
        print("")


def animate():
    for c in itertools.cycle(["|", "/", "-", "\\"]):
        if done:
            break
        sys.stdout.write("\r " + c)
        sys.stdout.flush()
        time.sleep(0.1)


def stop_loading_animation():
    global done
    done = True


def dictionary(word):
    url = "https://od-api.oxforddictionaries.com:443/api/v1/entries/en/" + word.lower()
    check_config_dir()

    app_id, app_key = read_config_file()
    headers = {"app_id": app_id, "app_key": app_key}
    try:
        response = requests.get(url, headers=headers)
        stop_loading_animation()
        print("")
    except ConnectionError:
        print("Unable to connect. Please check your internet connection.")
    if response.status_code == 200:
        parsed_response, examples = parse_dictionary_response(response)
        pretty_print_definition(word, parsed_response, examples)
        return parsed_response
    elif response.status_code == 404:
        print("No definition found. Please check the spelling!!")
        exit()
    if response.status_code == 403:
        cprint(
            "You have reached the api limit. This may happen because the api keys",
            "are shared among multiple users. You can either create your own free personal api key on https://developer.oxforddictionaries.com",
            " and paste them in ~/.pyvoc/pyvoc.config. or you can delete the the config file(~/.pyvoc/pyvoc.config) and run `pyvoc word`",
            "to get a new shared api key",
        )
    elif response.status_code == 500:
        print("Internal error. Error occured while processing the data")
        exit()
    else:
        print("Something went wrong. Check after some time.")
        exit()


def main():
    parser = argparse.ArgumentParser(
        description="command line dictionary and vocabulary building tool"
    )
    parser.add_argument("word", help="give meaning of WORD")
    parser.add_argument(
        "-a", "--add", action="store_true", help="add WORD to vocabulary group"
    )
    parser.add_argument(
        "-g", help="{optional} group no.(1-50) to add the WORD to", type=int
    )
    parser.add_argument(
        "-r",
        "--revise",
        action="store_true",
        help="revise a vocabulary group (WORD is group number).",
    )
    parser.add_argument(
        "-q",
        "--quiz",
        type=int,
        help="starts quiz, WORD is group no. and QUIZ is no. of questions",
    )
    parser.add_argument(
        "-l", "--list", action="store_true", help="list all user made vocabulary groups"
    )
    args = parser.parse_args()
    t = threading.Thread(target=animate)
    t.start()

    if args.revise:
        revise_vocab(args.word)
        exit()
    if args.quiz:
        cprint("\n\t\tStarting Quiz", color="red", attrs=["bold", "reverse"])
        quiz(int(args.word), args.quiz)
        exit()
    parsed_response = dictionary(args.word)
    if args.add:
        add_word_to_vocab(args.word.lower(), parsed_response, args.g)
    if args.list:
        list_all_groups()


if __name__ == "__main__":
    main()



import os
import json
import configparser

from termcolor import cprint
import colorama
import requests

colorama.init()

home_dir = os.path.expanduser("~")
config = configparser.ConfigParser()


def config_dir_path():
    return os.path.join(home_dir, ".pyvoc")


config_file_path = os.path.join(config_dir_path(), "pyvoc.config")


def check_config_dir():
    try:
        path = config_dir_path()
        # makes path recursively. returns None if already exist.
        os.makedirs(path, exist_ok=True)
        if not os.path.isfile(os.path.join(path, "group51.json")):
            with open(os.path.join(path, "group51.json"), "w") as f:
                json.dump({}, f)
        if not os.path.isfile(os.path.join(path, "group101.json")):
            response = requests.get(
                "https://raw.githubusercontent.com/makkoncept/definitions/master/group101.json"
            )
            with open(os.path.join(path, "group101.json"), "w") as f:
                json.dump(response.json(), f)
        if not os.path.isfile(os.path.join(path, "group102.json")):
            response = requests.get(
                "https://raw.githubusercontent.com/makkoncept/definitions/master/group102.json"
            )
            with open(os.path.join(path, "group102.json"), "w") as f:
                json.dump(response.json(), f)
        if not os.path.isfile(os.path.join(path, "group103.json")):
            response = requests.get(
                "https://raw.githubusercontent.com/makkoncept/definitions/master/group103.json"
            )
            with open(os.path.join(path, "group103.json"), "w") as f:
                json.dump(response.json(), f)
        if not os.path.isfile(os.path.join(path, "options.json")):
            response = requests.get(
                "https://raw.githubusercontent.com/makkoncept/doptions/master/options.json"
            )
            with open(os.path.join(path, "options.json"), "w") as f:
                json.dump(response.json(), f)

        if not os.path.isfile(os.path.join(path, "usergroups.json")):
            cprint("Creating necessary config files", color="yellow")
            with open(os.path.join(path, "usergroups.json"), "w") as f:
                json.dump({}, f)
        if not os.path.isfile(os.path.join(path, "defaultgroups.json")):
            with open(os.path.join(path, "defaultgroups.json"), "w") as f:
                json.dump({51: 0}, f)
        if not os.path.isfile(os.path.join(path, "all_words.json")):
            with open(os.path.join(path, "all_words.json"), "w") as f:
                json.dump({}, f)
        if not os.path.isfile(os.path.join(path, "pyvoc.config")):
            create_config_file()

    except IOError:
        print("Error occured while creating config files.")
        ()


def create_config_file():
    app_id, app_key = get_api_keys()
    config["API"] = {"app_id": app_id, "app_key": app_key}
    with open(config_file_path, "w") as f:
        config.write(f)


def read_config_file():
    config.read(config_file_path)
    return config["API"]["app_id"], config["API"]["app_key"]


def get_api_keys():
    url = "https://api.jsonbin.io/b/5c38856081fe89272a89a284/latest"
    # url = "https://api.jsonbin.io/b/5c31d08c05d34b26f202e4a5/latest"
    headers = {
        "secret-key": "$2a$10$B613zIvPJf5QQf.qDAHR0Op2DBthskxT90hltZ.UhsZA0o4Kqio2."
    }
    response = requests.get(url, headers=headers)
    print("getting api keys. please handle with care!")
    if response.status_code != 200:
        cprint("cannot get api key.")
        exit()
    json_response = response.json()
    keys = list(json_response.keys())
    key_count = 0
    api_id = None
    api_key = None
    backup = False
    for key in keys:
        key_count += 1
        if json_response[key]["count"] < 30:
            api_id = json_response[key]["app_id"]
            api_key = json_response[key]["app_key"]
            json_response[key]["count"] += 1
            break

    if key_count == len(keys) and api_id is None:
        # backup
        backup = True
        url = "https://api.jsonbin.io/b/5c3887932c87fa27306c4be2/latest"
        response = requests.get(url, headers=headers)
        # print("getting api keys")
        if response.status_code != 200:
            cprint("cannot get api key.")
            exit()
        json_response = response.json()
        keys = list(json_response.keys())
        key_count = 0
        api_id = None
        api_key = None
        for key in keys:
            key_count += 1
            if json_response[key]["count"] < 30:
                api_id = json_response[key]["app_id"]
                api_key = json_response[key]["app_key"]
                json_response[key]["count"] += 1
                break

    if backup is True:
        put_url = "https://api.jsonbin.io/b/5c3887932c87fa27306c4be2"
    else:
        put_url = "https://api.jsonbin.io/b/5c38856081fe89272a89a284"
    update_key_count(put_url, json_response)
    return api_id, api_key


def update_key_count(url, json_response):
    headers = {
        "Content-type": "application/json",
        "secret-key": "$2a$10$B613zIvPJf5QQf.qDAHR0Op2DBthskxT90hltZ.UhsZA0o4Kqio2.",
        "versioning": "false",
    }
    requests.put(url, json=json_response, headers=headers)




import json
import os

from termcolor import cprint
import colorama


from pyvoc.check_config import check_config_dir, config_dir_path
from pyvoc import pyvoc

colorama.init()


def add_new_vocab_group(group_number):
    pyvoc.stop_loading_animation()
    cprint("creating ", color="cyan", attrs=["bold"], end="")
    cprint("group Number {}...".format(group_number), color="green")

    with open(
        os.path.join(config_dir_path(), "group" + str(group_number) + ".json"), "w"
    ) as f:
        json.dump({}, f)


def validate_group_number(group_number):
    if group_number > 50 or group_number < 1:
        pyvoc.stop_loading_animation()
        cprint(
            "group number {} is not available. Choose from 1-50".format(group_number),
            color="red",
            attrs=["bold"],
        )
        exit()


def counter_increment(group_number):
    if not group_number:
        path = os.path.join(config_dir_path(), "defaultgroups.json")
        with open(path, "r") as f:
            content = json.load(f)
        latest_group_number = 50 + len(content)
        if latest_group_number > 100:
            cprint("Sorry! cannot add more words", color="red")
        if content[str(latest_group_number)] > 60:
            add_new_vocab_group(latest_group_number + 1)
            content[str(latest_group_number + 1)] = 1
            group_number = latest_group_number + 1
        else:
            content[str(latest_group_number)] += 1
            group_number = latest_group_number
        with open(path, "w") as f:
            json.dump(content, f)
        return group_number
    else:
        path = os.path.join(config_dir_path(), "usergroups.json")
        with open(path, "r") as f:
            content = json.load(f)
        group_number = group_number
        if str(group_number) not in content:
            content[str(group_number)] = 1
        elif content[str(group_number)] > 60:
            cprint(
                "cannot add more words to group number {}".format(group_number),
                color="yellow",
            )
            exit()
        else:
            content[str(group_number)] += 1
        with open(path, "w") as f:
            json.dump(content, f)
        return


def check_duplicity(word, group_number):
    if not group_number:
        path = os.path.join(config_dir_path(), "defaultgroups.json")
        with open(path, "r") as f:
            group_number = len(json.load(f)) + 50
        group_path = os.path.join(
            config_dir_path(), "group" + str(group_number) + ".json"
        )
        with open(group_path, "r") as f:
            content = json.load(f)
        if word in content:
            cprint(
                "Word already in group number {}. Choose a different group".format(
                    group_number
                ),
                color="yellow",
            )
            exit()  # maybe add a prompt to accept group number instead of exit
    else:
        group_path = os.path.join(
            config_dir_path(), "group" + str(group_number) + ".json"
        )
        with open(group_path, "r") as f:
            content = json.load(f)
        if word in content:
            cprint(
                "Word already in group number {}. Choose a different group".format(
                    group_number
                ),
                color="yellow",
            )
            exit()  # maybe add a prompt to accept group number instead of exit


def add_word_to_vocab(word, parsed_response, group_number=None):
    check_config_dir()
    config_path = config_dir_path()
    definition = {word: parsed_response}
    if not group_number:
        check_duplicity(word, group_number=False)
        group_number = counter_increment(group_number=False)
        group_path = os.path.join(
            config_dir_path(), "group" + str(group_number) + ".json"
        )
        pyvoc.stop_loading_animation()
        cprint("\nwriting to vocabulary group...", color="yellow")
        with open(group_path, "r") as f:
            content = json.load(f)
        content.update(definition)
        with open(group_path, "w") as f:
            json.dump(content, f, ensure_ascii=False)

    else:
        validate_group_number(group_number)
        group_path = os.path.join(
            config_dir_path(), "group" + str(group_number) + ".json"
        )
        if not os.path.isfile(group_path):
            add_new_vocab_group(group_number)
        pyvoc.stop_loading_animation()
        check_duplicity(word, group_number)
        counter_increment(group_number)
        cprint("writing to vocabulary group...", color="yellow")
        with open(group_path, "r") as f:
            content = json.load(f)
        content.update(definition)
        with open(group_path, "w") as f:
            json.dump(content, f, ensure_ascii=False)
    with open(os.path.join(config_path, "all_words.json"), "r") as f:
        all_words = json.load(f)
    all_words.update({word: True})
    with open(os.path.join(config_path, "all_words.json"), "w") as f:
        json.dump(all_words, f)

    cprint("word added to ", color="green", end="")
    cprint("group number {}".format(group_number), color="cyan", attrs=["bold"])


def list_all_groups():
    usergroups_path = os.path.join(config_dir_path(), "usergroups.json")
    defaultgroups_path = os.path.join(config_dir_path(), "defaultgroups.json")
    with open(usergroups_path, "r") as f:
        user_group_numbers = json.load(f)
    pyvoc.stop_loading_animation()
    print("")
    cprint("USER GROUPS", color="cyan", on_color="on_grey")
    cprint("Group no.", color="green", end=" " * (14 - len("Group no")))
    cprint("No. of words")
    for group in user_group_numbers:
        cprint(group, color="green", end=" " * (15 - len(str(group))))
        cprint(str(user_group_numbers[group]))
    with open(defaultgroups_path, "r") as f:
        default_group_numbers = json.load(f)
    print("")
    cprint("DEFAULT GROUP", color="cyan", on_color="on_grey")
    cprint("Group no.", color="green", end=" " * (14 - len("Group no")))
    cprint("No. of words")
    for group in default_group_numbers:
        cprint(group, color="green", end=" " * (15 - len(str(group))))
        cprint(str(default_group_numbers[group]))




from setuptools import setup


with open("README.md") as infile:
    long_description = infile.read()


setup(
    name="pyvoc",
    version="1.1.0",
    packages=["pyvoc"],
    description="cross-platform dictionary and vocabulary building command line tool.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="http://github.com/makkoncept/pyvoc",
    author="Mayank Nader",
    author_mail="nader.mayank@gmail.com",
    install_requires=["requests", "termcolor", "colorama"],
    entry_points={"console_scripts": ["pyvoc=pyvoc.pyvoc:main"]},
    include_package_data=True,
)
